@@ -24,7 +24,7 @@
 #include "config.h"
 #endif
 
-#define VERSION "1.9.6"
+#define VERSION "1.9.7"
 
 #define BUFFER_SIZE_MAX 8192
 
@@ -33,6 +33,8 @@
 
 #define JACK_CLIENT_NAME_SIZE 64
 #define JACK_MESSAGE_SIZE 256
+#define JACK_UUID_SIZE 32
+#define JACK_SESSION_COMMAND_SIZE 256
 
 #ifndef PORT_NUM
 #define PORT_NUM 2048
@@ -71,6 +73,7 @@
 #define FREEWHEEL_DRIVER_TIMEOUT 10     // in sec
 #define DRIVER_TIMEOUT_FACTOR    10     
 
+
 #define NO_PORT   0xFFFE
 
 #define EMPTY   0xFFFD
