@@ -12,7 +12,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with this program; if not, write to the Free Software 
+along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 */
@@ -36,8 +36,8 @@ namespace Jack
 
 JackDebugClient::JackDebugClient(JackClient * client)
 {
-    fTotalPortNumber = 1;	// The total number of port opened and maybe closed. Historical view.
-    fOpenPortNumber = 0;	// The current number of opened port.
+    fTotalPortNumber = 1;       // The total number of port opened and maybe closed. Historical view.
+    fOpenPortNumber = 0;        // The current number of opened port.
     fIsActivated = 0;
     fIsDeactivated = 0;
     fIsClosed = 0;
@@ -78,9 +78,9 @@ JackDebugClient::~JackDebugClient()
     delete fClient;
 }
 
-int JackDebugClient::Open(const char* server_name, const char* name, jack_options_t options, jack_status_t* status)
+int JackDebugClient::Open(const char* server_name, const char* name, int uuid, jack_options_t options, jack_status_t* status)
 {
-    int res = fClient->Open(server_name, name, options, status);
+    int res = fClient->Open(server_name, name, uuid, options, status);
     char provstr[256];
     char buffer[256];
     time_t curtime;
@@ -116,17 +116,17 @@ int JackDebugClient::Close()
 void JackDebugClient::CheckClient(const char* function_name) const
 {
     *fStream << "CheckClient : " << function_name << ", calling thread : " << pthread_self() << endl;
-    
+
     if (fIsClosed > 0)  {
-        *fStream << "!!! ERROR !!! : Accessing a client '" << fClientName << "' already closed " << "from " << function_name << endl; 
+        *fStream << "!!! ERROR !!! : Accessing a client '" << fClientName << "' already closed " << "from " << function_name << endl;
         *fStream << "This is likely to cause crash !'" << endl;
     #ifdef __APPLE__
        // Debugger();
-    #endif 
+    #endif
     }
 }
 
-pthread_t JackDebugClient::GetThreadID()
+jack_native_thread_t JackDebugClient::GetThreadID()
 {
     CheckClient("GetThreadID");
     return fClient->GetThreadID();
@@ -210,8 +210,8 @@ int JackDebugClient::PortUnRegister(jack_port_id_t port_index)
     int res = fClient->PortUnRegister(port_index);
     fOpenPortNumber--;
     int i;
-    for (i = (fTotalPortNumber - 1); i >= 0; i--) {	// We search the record into the history
-        if (fPortList[i].idport == port_index) {		// We found the last record
+    for (i = (fTotalPortNumber - 1); i >= 0; i--) {     // We search the record into the history
+        if (fPortList[i].idport == port_index) {                // We found the last record
             if (fPortList[i].IsUnregistered != 0)
                 *fStream << "!!! ERROR !!! : '" << fClientName << "' id deregistering port '" << fPortList[i].name << "' that have already been unregistered !" << endl;
             fPortList[i].IsUnregistered++;
@@ -233,8 +233,8 @@ int JackDebugClient::PortConnect(const char* src, const char* dst)
         *fStream << "!!! ERROR !!! Trying to connect a port ( " << src << " to " << dst << ") while the client has not been activated !" << endl;
     int i;
     int res = fClient->PortConnect( src, dst);
-    for (i = (fTotalPortNumber - 1); i >= 0; i--) {	// We search the record into the history
-        if (strcmp(fPortList[i].name, src) == 0) {	// We found the last record in sources
+    for (i = (fTotalPortNumber - 1); i >= 0; i--) {     // We search the record into the history
+        if (strcmp(fPortList[i].name, src) == 0) {      // We found the last record in sources
             if (fPortList[i].IsUnregistered != 0)
                 *fStream << "!!! ERROR !!! Connecting port " << src << " previoulsy unregistered !" << endl;
             fPortList[i].IsConnected++;
@@ -293,8 +293,8 @@ int JackDebugClient::PortDisconnect(jack_port_id_t src)
         *fStream << "!!! ERROR !!! : Trying to disconnect port  " << src << " while that client has not been activated !" << endl;
     int res = fClient->PortDisconnect(src);
     int i;
-    for (i = (fTotalPortNumber - 1); i >= 0; i--) {		// We search the record into the history
-        if (fPortList[i].idport == src) {				// We found the record in sources
+    for (i = (fTotalPortNumber - 1); i >= 0; i--) {             // We search the record into the history
+        if (fPortList[i].idport == src) {                               // We found the record in sources
             if (fPortList[i].IsUnregistered != 0)
                 *fStream << "!!! ERROR !!! : Disconnecting port  " << src << " previoulsy unregistered !" << endl;
             fPortList[i].IsConnected--;
@@ -428,7 +428,7 @@ void JackDebugClient::OnInfoShutdown(JackInfoShutdownCallback callback, void *ar
     CheckClient("OnInfoShutdown");
     fClient->OnInfoShutdown(callback, arg);
 }
-    
+
 int JackDebugClient::TimeCallback(jack_nframes_t nframes, void *arg)
 {
     JackDebugClient* client = (JackDebugClient*)arg;
@@ -505,6 +505,54 @@ int JackDebugClient::SetPortRenameCallback(JackPortRenameCallback callback, void
     return fClient->SetPortRenameCallback(callback, arg);
 }
 
+int JackDebugClient::SetSessionCallback(JackSessionCallback callback, void *arg)
+{
+    CheckClient("SetSessionCallback");
+    return fClient->SetSessionCallback(callback, arg);
+}
+
+int JackDebugClient::SetLatencyCallback(JackLatencyCallback callback, void *arg)
+{
+    CheckClient("SetLatencyCallback");
+    return fClient->SetLatencyCallback(callback, arg);
+}
+
+jack_session_command_t* JackDebugClient::SessionNotify(const char* target, jack_session_event_type_t type, const char* path)
+{
+    CheckClient("SessionNotify");
+    return fClient->SessionNotify(target, type, path);
+}
+
+int JackDebugClient::SessionReply(jack_session_event_t* ev)
+{
+    CheckClient("SessionReply");
+    return fClient->SessionReply(ev);
+}
+
+char* JackDebugClient::GetUUIDForClientName(const char* client_name)
+{
+    CheckClient("GetUUIDForClientName");
+    return fClient->GetUUIDForClientName(client_name);
+}
+
+char* JackDebugClient::GetClientNameByUUID(const char* uuid)
+{
+    CheckClient("GetClientNameByUUID");
+    return fClient->GetClientNameByUUID(uuid);
+}
+
+int JackDebugClient::ReserveClientName(const char* client_name, const char* uuid)
+{
+    CheckClient("ReserveClientName");
+    return fClient->ReserveClientName(client_name, uuid);
+}
+
+int JackDebugClient::ClientHasSessionCallback(const char* client_name)
+{
+    CheckClient("ClientHasSessionCallback");
+    return fClient->ClientHasSessionCallback(client_name);
+}
+
 JackClientControl* JackDebugClient::GetClientControl() const
 {
     CheckClient("GetClientControl");
