@@ -32,7 +32,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 jack_driver_desc_t * jackctl_driver_get_desc(jackctl_driver_t * driver);
 
-SERVER_EXPORT void jack_print_driver_options (jack_driver_desc_t* desc, FILE* file)
+EXPORT void jack_print_driver_options (jack_driver_desc_t* desc, FILE* file)
 {
     unsigned long i;
     char arg_default[JACK_DRIVER_PARAM_STRING_MAX + 1];
@@ -75,7 +75,7 @@ jack_print_driver_param_usage (jack_driver_desc_t * desc, unsigned long param, F
     fprintf (file, "%s\n", desc->params[param].long_desc);
 }
 
-SERVER_EXPORT void jack_free_driver_params(JSList * driver_params)
+EXPORT void jack_free_driver_params(JSList * driver_params)
 {
     JSList *node_ptr = driver_params;
     JSList *next_node_ptr;
@@ -119,7 +119,7 @@ jack_parse_driver_params (jack_driver_desc_t * desc, int argc, char* argv[], JSL
                      desc->name);
         }
 
-        printf ("Parameters for driver '%s' (all parameters are optional):\n", desc->name);
+        jack_log("Parameters for driver '%s' (all parameters are optional):", desc->name);
         jack_print_driver_options (desc, stdout);
         return 1;
     }
@@ -228,7 +228,7 @@ jack_parse_driver_params (jack_driver_desc_t * desc, int argc, char* argv[], JSL
     return 0;
 }
 
-SERVER_EXPORT int
+EXPORT int
 jackctl_parse_driver_params (jackctl_driver *driver_ptr, int argc, char* argv[])
 {
     struct option * long_options;
@@ -263,7 +263,7 @@ jackctl_parse_driver_params (jackctl_driver *driver_ptr, int argc, char* argv[])
                      desc->name);
         }
 
-        printf ("Parameters for driver '%s' (all parameters are optional):\n", desc->name);
+        jack_log("Parameters for driver '%s' (all parameters are optional):", desc->name);
         jack_print_driver_options (desc, stdout);
         return 1;
     }
@@ -436,10 +436,10 @@ jack_get_descriptor (JSList * drivers, const char * sofile, const char * symbol)
 
 #ifdef WIN32
     if ((so_get_descriptor == NULL) && (dlerr = GetLastError()) != 0) {
-        jack_log("jack_get_descriptor : dll is not a driver, err = %ld", dlerr);
+        jack_error("jack_get_descriptor : dll is not a driver, err = %ld", dlerr);
 #else
     if ((so_get_descriptor == NULL) && (dlerr = dlerror ()) != NULL) {
-        jack_log("jack_get_descriptor err = %s", dlerr);
+        jack_error("jack_get_descriptor err = %s", dlerr);
 #endif
 
         UnloadDriverModule(dlhandle);
@@ -563,6 +563,8 @@ jack_drivers_load (JSList * drivers) {
         desc = jack_get_descriptor (drivers, filedata.cFileName, "driver_get_descriptor");
         if (desc) {
             driver_list = jack_slist_append (driver_list, desc);
+        } else {
+            jack_error ("jack_get_descriptor returns null for \'%s\'", filedata.cFileName);
         }
 
     } while (FindNextFile(file, &filedata));
@@ -617,8 +619,11 @@ jack_drivers_load (JSList * drivers) {
         }
 
         desc = jack_get_descriptor (drivers, dir_entry->d_name, "driver_get_descriptor");
+
         if (desc) {
             driver_list = jack_slist_append (driver_list, desc);
+        } else {
+            jack_error ("jack_get_descriptor returns null for \'%s\'", dir_entry->d_name);
         }
     }
 
@@ -687,6 +692,8 @@ jack_internals_load (JSList * internals) {
         desc = jack_get_descriptor (internals, filedata.cFileName, "jack_get_descriptor");
         if (desc) {
             driver_list = jack_slist_append (driver_list, desc);
+        } else {
+            jack_error ("jack_get_descriptor returns null for \'%s\'", filedata.cFileName);
         }
 
     } while (FindNextFile(file, &filedata));
@@ -743,6 +750,8 @@ jack_internals_load (JSList * internals) {
         desc = jack_get_descriptor (internals, dir_entry->d_name, "jack_get_descriptor");
         if (desc) {
             driver_list = jack_slist_append (driver_list, desc);
+        } else {
+            jack_error ("jack_get_descriptor returns null for \'%s\'", dir_entry->d_name);
         }
     }
 
@@ -762,9 +771,9 @@ jack_internals_load (JSList * internals) {
 
 #endif
 
-Jack::JackDriverClientInterface* JackDriverInfo::Open(jack_driver_desc_t* driver_desc, 
-                                                    Jack::JackLockedEngine* engine, 
-                                                    Jack::JackSynchro* synchro, 
+Jack::JackDriverClientInterface* JackDriverInfo::Open(jack_driver_desc_t* driver_desc,
+                                                    Jack::JackLockedEngine* engine,
+                                                    Jack::JackSynchro* synchro,
                                                     const JSList* params)
 {
 #ifdef WIN32
@@ -774,7 +783,7 @@ Jack::JackDriverClientInterface* JackDriverInfo::Open(jack_driver_desc_t* driver
 #endif
 
     fHandle = LoadDriverModule (driver_desc->file);
-    
+
     if (fHandle == NULL) {
 #ifdef WIN32
         if ((errstr = GetLastError ()) != 0) {
@@ -800,8 +809,14 @@ Jack::JackDriverClientInterface* JackDriverInfo::Open(jack_driver_desc_t* driver
         jack_error("no initialize function in shared object %s\n", driver_desc->file);
         return NULL;
     }
-    
+
     fBackend = fInitialize(engine, synchro, params);
     return fBackend;
 }
 
+JackDriverInfo::~JackDriverInfo()
+{
+    delete fBackend;
+    if (fHandle)
+        UnloadDriverModule(fHandle);
+}
