@@ -7,12 +7,12 @@
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
@@ -25,7 +25,6 @@
 #include <string.h>
 #include <errno.h>
 #include "JackCompilerDeps.h"
-#include "types.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -35,27 +34,27 @@ extern "C"
 #define LOG_LEVEL_INFO   1
 #define LOG_LEVEL_ERROR  2
 
-    SERVER_EXPORT void jack_error(const char *fmt, ...);
+    EXPORT void jack_error(const char *fmt, ...);
 
-    SERVER_EXPORT void jack_info(const char *fmt, ...);
+    EXPORT void jack_info(const char *fmt, ...);
 
     // like jack_info() but only if verbose mode is enabled
-    SERVER_EXPORT void jack_log(const char *fmt, ...);
+    EXPORT void jack_log(const char *fmt, ...);
 
-    SERVER_EXPORT extern void (*jack_error_callback)(const char *desc);
-    SERVER_EXPORT extern void (*jack_info_callback)(const char *desc);
-    
-    SERVER_EXPORT extern void default_jack_error_callback(const char *desc);
-    SERVER_EXPORT extern void default_jack_info_callback(const char *desc);
-    
-    SERVER_EXPORT extern void silent_jack_error_callback(const char *desc);
-    SERVER_EXPORT extern void silent_jack_info_callback(const char *desc);
+    EXPORT extern void (*jack_error_callback)(const char *desc);
+    EXPORT extern void (*jack_info_callback)(const char *desc);
+
+    EXPORT extern void default_jack_error_callback(const char *desc);
+    EXPORT extern void default_jack_info_callback(const char *desc);
+
+    EXPORT extern void silent_jack_error_callback(const char *desc);
+    EXPORT extern void silent_jack_info_callback(const char *desc);
 
     typedef void (* jack_log_function_t)(int level, const char *message);
 
     void jack_log_function(int level, const char *message);
-   
-    SERVER_EXPORT int set_threaded_log_function();
+
+    EXPORT int set_threaded_log_function();
 
 #ifdef __cplusplus
 }
