@@ -57,7 +57,7 @@ int JackLibGlobals::fClientCount = 0;
 
 jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t options, jack_status_t* status)
 {
-    jack_varargs_t va;		/* variable arguments */
+    jack_varargs_t va;          /* variable arguments */
     jack_status_t my_status;
     JackClient* client;
  
@@ -68,8 +68,8 @@ jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t optio
 
     jack_log("jack_client_new %s", client_name);
    
-    if (status == NULL)			/* no status from caller? */
-        status = &my_status;	/* use local status word */
+    if (status == NULL)         /* no status from caller? */
+        status = &my_status;    /* use local status word */
     *status = (jack_status_t)0;
 
     /* validate parameters */
@@ -96,7 +96,7 @@ jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t optio
         client = new JackLibClient(GetSynchroTable());
     }
 
-    int res = client->Open(va.server_name, client_name, options, status);
+    int res = client->Open(va.server_name, client_name, va.session_id, options, status);
     if (res < 0) {
         delete client;
         JackLibGlobals::Destroy(); // jack library destruction
@@ -149,7 +149,7 @@ jack_client_t* jack_client_open_aux(const char* client_name, jack_options_t opti
         client = new JackLibClient(GetSynchroTable());
     }
 
-    int res = client->Open(va.server_name, client_name, options, status);
+    int res = client->Open(va.server_name, client_name, va.session_id, options, status);
     if (res < 0) {
         delete client;
         JackLibGlobals::Destroy(); // jack library destruction
