@@ -53,7 +53,7 @@ using namespace Jack;
 
 jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t options, jack_status_t* status)
 {
-    jack_varargs_t va;		/* variable arguments */
+    jack_varargs_t va;          /* variable arguments */
     jack_status_t my_status;
     JackClient* client;
 
@@ -63,9 +63,9 @@ jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t optio
     }
 
     jack_log("jack_client_new %s", client_name);
- 
-    if (status == NULL)			/* no status from caller? */
-        status = &my_status;	/* use local status word */
+
+    if (status == NULL)         /* no status from caller? */
+        status = &my_status;    /* use local status word */
     *status = (jack_status_t)0;
 
     /* validate parameters */
@@ -77,20 +77,20 @@ jack_client_t* jack_client_new_aux(const char* client_name, jack_options_t optio
 
     /* parse variable arguments */
     jack_varargs_init(&va);
- 
+
     if (!JackServerGlobals::Init()) { // jack server initialisation
         int my_status1 = (JackFailure | JackServerError);
         *status = (jack_status_t)my_status1;
         return NULL;
     }
- 
+
     if (JACK_DEBUG) {
         client = new JackDebugClient(new JackInternalClient(JackServerGlobals::fInstance, GetSynchroTable())); // Debug mode
     } else {
         client = new JackInternalClient(JackServerGlobals::fInstance, GetSynchroTable());
     }
 
-    int res = client->Open(va.server_name, client_name, options, status);
+    int res = client->Open(va.server_name, client_name, va.session_id, options, status);
     if (res < 0) {
         delete client;
         JackServerGlobals::Destroy(); // jack server destruction
@@ -114,7 +114,7 @@ jack_client_t* jack_client_open_aux(const char* client_name, jack_options_t opti
     }
 
     jack_log("jack_client_open %s", client_name);
- 
+
     if (status == NULL)			/* no status from caller? */
         status = &my_status;	/* use local status word */
     *status = (jack_status_t)0;
@@ -128,20 +128,20 @@ jack_client_t* jack_client_open_aux(const char* client_name, jack_options_t opti
 
     /* parse variable arguments */
     jack_varargs_parse(options, ap, &va);
- 
+
     if (!JackServerGlobals::Init()) { // jack server initialisation
         int my_status1 = (JackFailure | JackServerError);
         *status = (jack_status_t)my_status1;
         return NULL;
     }
- 
+
     if (JACK_DEBUG) {
         client = new JackDebugClient(new JackInternalClient(JackServerGlobals::fInstance, GetSynchroTable())); // Debug mode
     } else {
         client = new JackInternalClient(JackServerGlobals::fInstance, GetSynchroTable());
     }
 
-    int res = client->Open(va.server_name, client_name, options, status);
+    int res = client->Open(va.server_name, client_name, va.session_id, options, status);
     if (res < 0) {
         delete client;
         JackServerGlobals::Destroy(); // jack server destruction
@@ -180,7 +180,7 @@ EXPORT int jack_client_close(jack_client_t* ext_client)
 {
 #ifdef __CLIENTDEBUG__
     JackGlobals::CheckContext("jack_client_close");
-#endif    
+#endif
     assert(JackGlobals::fOpenMutex);
     JackGlobals::fOpenMutex->Lock();
     int res = -1;
@@ -191,7 +191,7 @@ EXPORT int jack_client_close(jack_client_t* ext_client)
     } else {
         res = client->Close();
         delete client;
-        JackServerGlobals::Destroy();	// jack server destruction
+        JackServerGlobals::Destroy();   // jack server destruction
         jack_log("jack_client_close res = %d", res);
     }
     JackGlobals::fOpenMutex->Unlock();
@@ -200,7 +200,7 @@ EXPORT int jack_client_close(jack_client_t* ext_client)
 
 EXPORT int jack_get_client_pid(const char *name)
 {
-    return (JackServerGlobals::fInstance != NULL) 
+    return (JackServerGlobals::fInstance != NULL)
         ? JackServerGlobals::fInstance->GetEngine()->GetClientPID(name)
         : 0;
 }
