@@ -25,7 +25,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 $Id: net_driver.c,v 1.17 2006/04/16 20:16:10 torbenh Exp $
 */
 
-
 #include <math.h>
 #include <stdio.h>
 #include <memory.h>
@@ -48,8 +47,6 @@ $Id: net_driver.c,v 1.17 2006/04/16 20:16:10 torbenh Exp $
 #include <netinet/in.h>
 #endif
 
-#include "netjack.h"
-
 #ifdef __linux__
 #include "config.h"
 #endif
@@ -58,15 +55,11 @@ $Id: net_driver.c,v 1.17 2006/04/16 20:16:10 torbenh Exp $
 #include <samplerate.h>
 #endif
 
-#if HAVE_CELT
-#include <celt/celt.h>
-#endif
-
 #include "netjack.h"
 #include "netjack_packet.h"
 
 // JACK2
-#include "jack/control.h"
+#include "control.h"
 
 #define MIN(x,y) ((x)<(y) ? (x) : (y))
 
@@ -105,8 +98,8 @@ int netjack_wait( netjack_driver_state_t *netj )
 	netj->expected_framecnt += 1;
     } else {
 	// starting up.... lets look into the packetcache, and fetch the highest packet.
-	packet_cache_drain_socket( global_packcache, netj->sockfd );
-	if( packet_cache_get_highest_available_framecnt( global_packcache, &next_frame_avail ) ) {
+	packet_cache_drain_socket( netj->packcache, netj->sockfd );
+	if( packet_cache_get_highest_available_framecnt( netj->packcache, &next_frame_avail ) ) {
 	    netj->expected_framecnt = next_frame_avail;
 	    netj->expected_framecnt_valid = 1;
 	} else {
@@ -122,7 +115,7 @@ int netjack_wait( netjack_driver_state_t *netj )
     // then poll (have deadline calculated)
     // then drain socket, rinse and repeat.
     while(1) {
-	if( packet_cache_get_next_available_framecnt( global_packcache, netj->expected_framecnt, &next_frame_avail) ) {
+	if( packet_cache_get_next_available_framecnt( netj->packcache, netj->expected_framecnt, &next_frame_avail) ) {
 	    if( next_frame_avail == netj->expected_framecnt ) {
 		we_have_the_expected_frame = 1;
 		if( !netj->always_deadline )
@@ -133,13 +126,13 @@ int netjack_wait( netjack_driver_state_t *netj )
 	    break;
 	}
 
-	packet_cache_drain_socket( global_packcache, netj->sockfd );
+	packet_cache_drain_socket( netj->packcache, netj->sockfd );
     }
 
     // check if we know who to send our packets too.
     if (!netj->srcaddress_valid)
-	if( global_packcache->master_address_valid ) {
-	    memcpy (&(netj->syncsource_address), &(global_packcache->master_address), sizeof( struct sockaddr_in ) );
+	if( netj->packcache->master_address_valid ) {
+	    memcpy (&(netj->syncsource_address), &(netj->packcache->master_address), sizeof( struct sockaddr_in ) );
 	    netj->srcaddress_valid = 1;
 	}
 
@@ -161,7 +154,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 	else
 		netj->time_to_deadline = 0;
 
-	packet_cache_retreive_packet_pointer( global_packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize , &packet_recv_time_stamp);
+	packet_cache_retreive_packet_pointer( netj->packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize , &packet_recv_time_stamp);
 	pkthdr = (jacknet_packet_header *) netj->rx_buf;
 	packet_header_ntoh(pkthdr);
 	netj->deadline_goodness = (int)pkthdr->sync_state;
@@ -203,7 +196,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 	// lets check if we have the next packets, we will just run a cycle without data.
 	// in that case.
 
-	if( packet_cache_get_next_available_framecnt( global_packcache, netj->expected_framecnt, &next_frame_avail) )
+	if( packet_cache_get_next_available_framecnt( netj->packcache, netj->expected_framecnt, &next_frame_avail) )
 	{
 	    jack_nframes_t offset = next_frame_avail - netj->expected_framecnt;
 
@@ -221,7 +214,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 
 		// I also found this happening, when the packet queue, is too full.
 		// but wtf ? use a smaller latency. this link can handle that ;S
-		if( packet_cache_get_fill( global_packcache, netj->expected_framecnt ) > 80.0 )
+		if( packet_cache_get_fill( netj->packcache, netj->expected_framecnt ) > 80.0 )
 		    netj->next_deadline -= netj->period_usecs/2;
 
 
@@ -229,7 +222,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 		// the diff is too high. but we have a packet in the future.
 		// lets resync.
 		netj->expected_framecnt = next_frame_avail;
-		packet_cache_retreive_packet_pointer( global_packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize, NULL );
+		packet_cache_retreive_packet_pointer( netj->packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize, NULL );
 		pkthdr = (jacknet_packet_header *) netj->rx_buf;
 		packet_header_ntoh(pkthdr);
 		//netj->deadline_goodness = 0;
@@ -257,7 +250,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 		// i will make the packet cache drop redundant packets,
 		// that have already been retreived.
 		//
-		if( packet_cache_get_highest_available_framecnt( global_packcache, &next_frame_avail) ) {
+		if( packet_cache_get_highest_available_framecnt( netj->packcache, &next_frame_avail) ) {
 		    if( next_frame_avail == (netj->expected_framecnt - 1) ) {
 			// Ok. the last packet is there now.
 			// and it had not been retrieved.
@@ -277,9 +270,9 @@ int netjack_wait( netjack_driver_state_t *netj )
 
 		// But now we can check for any new frame available.
 		//
-		if( packet_cache_get_highest_available_framecnt( global_packcache, &next_frame_avail) ) {
+		if( packet_cache_get_highest_available_framecnt( netj->packcache, &next_frame_avail) ) {
 		    netj->expected_framecnt = next_frame_avail;
-		    packet_cache_retreive_packet_pointer( global_packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize, NULL );
+		    packet_cache_retreive_packet_pointer( netj->packcache, netj->expected_framecnt, (char **) &(netj->rx_buf), netj->rx_bufsize, NULL );
 		    pkthdr = (jacknet_packet_header *) netj->rx_buf;
 		    packet_header_ntoh(pkthdr);
 		    netj->deadline_goodness = pkthdr->sync_state;
@@ -300,7 +293,7 @@ int netjack_wait( netjack_driver_state_t *netj )
 		    // reply address changes port.
 		    if (netj->num_lost_packets > 200 ) {
 			netj->srcaddress_valid = 0;
-			packet_cache_reset_master_address( global_packcache );
+			packet_cache_reset_master_address( netj->packcache );
 		    }
 		}
 	    }
@@ -369,6 +362,21 @@ void netjack_attach( netjack_driver_state_t *netj )
     int port_flags;
 
 
+    if( netj->bitdepth == CELT_MODE )
+    {
+#if HAVE_CELT
+#if HAVE_CELT_API_0_7 || HAVE_CELT_API_0_8 || HAVE_CELT_API_0_11
+	    celt_int32 lookahead;
+	    netj->celt_mode = celt_mode_create( netj->sample_rate, netj->period_size, NULL );
+#else
+	    celt_int32_t lookahead;
+	    netj->celt_mode = celt_mode_create( netj->sample_rate, 1, netj->period_size, NULL );
+#endif
+	    celt_mode_info( netj->celt_mode, CELT_GET_LOOKAHEAD, &lookahead );
+	    netj->codec_latency = 2*lookahead;
+#endif
+    }
+
     if (netj->handle_transport_sync)
         jack_set_sync_callback(netj->client, (JackSyncCallback) net_driver_sync_cb, NULL);
 
@@ -390,17 +398,13 @@ void netjack_attach( netjack_driver_state_t *netj )
 
 	if( netj->bitdepth == CELT_MODE ) {
 #if HAVE_CELT
-#if HAVE_CELT_API_0_7
-	    celt_int32 lookahead;
-	    CELTMode *celt_mode = celt_mode_create( netj->sample_rate, netj->period_size, NULL );
-	    netj->capture_srcs = jack_slist_append(netj->capture_srcs, celt_decoder_create( celt_mode, 1, NULL ) );
+#if HAVE_CELT_API_0_11
+        netj->capture_srcs = jack_slist_append(netj->capture_srcs, celt_decoder_create_custom( netj->celt_mode, 1, NULL ) );
+#elif HAVE_CELT_API_0_7 || HAVE_CELT_API_0_8
+	    netj->capture_srcs = jack_slist_append(netj->capture_srcs, celt_decoder_create( netj->celt_mode, 1, NULL ) );
 #else
-	    celt_int32_t lookahead;
-	    CELTMode *celt_mode = celt_mode_create( netj->sample_rate, 1, netj->period_size, NULL );
-	    netj->capture_srcs = jack_slist_append(netj->capture_srcs, celt_decoder_create( celt_mode ) );
+	    netj->capture_srcs = jack_slist_append(netj->capture_srcs, celt_decoder_create( netj->celt_mode ) );
 #endif
-	    celt_mode_info( celt_mode, CELT_GET_LOOKAHEAD, &lookahead );
-	    netj->codec_latency = 2*lookahead;
 #endif
 	} else {
 #if HAVE_SAMPLERATE
@@ -408,6 +412,7 @@ void netjack_attach( netjack_driver_state_t *netj )
 #endif
 	}
     }
+
     for (chn = netj->capture_channels_audio; chn < netj->capture_channels; chn++) {
         snprintf (buf, sizeof(buf) - 1, "capture_%u", chn + 1);
 
@@ -441,7 +446,10 @@ void netjack_attach( netjack_driver_state_t *netj )
             jack_slist_append (netj->playback_ports, port);
 	if( netj->bitdepth == CELT_MODE ) {
 #if HAVE_CELT
-#if HAVE_CELT_API_0_7
+#if HAVE_CELT_API_0_11
+        CELTMode *celt_mode = celt_mode_create( netj->sample_rate, netj->period_size, NULL );
+	    netj->playback_srcs = jack_slist_append(netj->playback_srcs, celt_decoder_create_custom( celt_mode, 1, NULL ) );
+#elif HAVE_CELT_API_0_7 || HAVE_CELT_API_0_8
 	    CELTMode *celt_mode = celt_mode_create( netj->sample_rate, netj->period_size, NULL );
 	    netj->playback_srcs = jack_slist_append(netj->playback_srcs, celt_encoder_create( celt_mode, 1, NULL ) );
 #else
@@ -479,7 +487,6 @@ void netjack_detach( netjack_driver_state_t *netj )
 {
     JSList * node;
 
-
     for (node = netj->capture_ports; node; node = jack_slist_next (node))
         jack_port_unregister (netj->client,
                               ((jack_port_t *) node->data));
@@ -487,29 +494,74 @@ void netjack_detach( netjack_driver_state_t *netj )
     jack_slist_free (netj->capture_ports);
     netj->capture_ports = NULL;
 
+    for (node = netj->capture_srcs; node; node = jack_slist_next (node))
+    {
+#if HAVE_CELT
+        if( netj->bitdepth == CELT_MODE )
+        {
+            CELTDecoder * decoder = node->data;
+            celt_decoder_destroy(decoder);
+        }
+        else
+#endif
+        {
+#if HAVE_SAMPLERATE
+            SRC_STATE * src = node->data;
+            src_delete(src);
+#endif
+        }
+    }
+    jack_slist_free (netj->capture_srcs);
+    netj->playback_srcs = NULL;
+
     for (node = netj->playback_ports; node; node = jack_slist_next (node))
         jack_port_unregister (netj->client,
                               ((jack_port_t *) node->data));
 
     jack_slist_free (netj->playback_ports);
     netj->playback_ports = NULL;
+
+    for (node = netj->playback_srcs; node; node = jack_slist_next (node))
+    {
+#if HAVE_CELT
+        if( netj->bitdepth == CELT_MODE )
+        {
+            CELTEncoder * encoder = node->data;
+            celt_encoder_destroy(encoder);
+        }
+        else
+#endif
+        {
+#if HAVE_SAMPLERATE
+            SRC_STATE * src = node->data;
+            src_delete(src);
+#endif
+        }
+    }
+    jack_slist_free (netj->playback_srcs);
+    netj->playback_srcs = NULL;
+
+#if HAVE_CELT
+        if( netj->bitdepth == CELT_MODE )
+		celt_mode_destroy(netj->celt_mode);
+#endif
 }
 
 
 netjack_driver_state_t *netjack_init (netjack_driver_state_t *netj,
 		jack_client_t * client,
-                const char *name,
-                unsigned int capture_ports,
-                unsigned int playback_ports,
-                unsigned int capture_ports_midi,
-                unsigned int playback_ports_midi,
-                jack_nframes_t sample_rate,
-                jack_nframes_t period_size,
-                unsigned int listen_port,
-                unsigned int transport_sync,
-                unsigned int resample_factor,
-                unsigned int resample_factor_up,
-                unsigned int bitdepth,
+        const char *name,
+        unsigned int capture_ports,
+        unsigned int playback_ports,
+        unsigned int capture_ports_midi,
+        unsigned int playback_ports_midi,
+        jack_nframes_t sample_rate,
+        jack_nframes_t period_size,
+        unsigned int listen_port,
+        unsigned int transport_sync,
+        unsigned int resample_factor,
+        unsigned int resample_factor_up,
+        unsigned int bitdepth,
 		unsigned int use_autoconfig,
 		unsigned int latency,
 		unsigned int redundancy,
@@ -574,8 +626,8 @@ void netjack_release( netjack_driver_state_t *netj )
     close( netj->sockfd );
     close( netj->outsockfd );
 
-    packet_cache_free( global_packcache );
-    global_packcache = NULL;
+    packet_cache_free( netj->packcache );
+    netj->packcache = NULL;
 }
 
 int
@@ -585,13 +637,7 @@ netjack_startup( netjack_driver_state_t *netj )
     struct sockaddr_in address;
     // Now open the socket, and wait for the first packet to arrive...
     netj->sockfd = socket (AF_INET, SOCK_DGRAM, 0);
-
 #ifdef WIN32
-    u_long parm = 1;
-    DWORD bufsize = 262144;
-    //ioctlsocket( netj->sockfd, FIONBIO, &parm );
-    setsockopt( netj->sockfd, SOL_SOCKET, SO_RCVBUF, (char *)&bufsize, sizeof(bufsize) );
-    setsockopt( netj->sockfd, SOL_SOCKET, SO_SNDBUF, (char *)&bufsize, sizeof(bufsize) );
     if (netj->sockfd == INVALID_SOCKET)
 #else
     if (netj->sockfd == -1)
@@ -622,70 +668,74 @@ netjack_startup( netjack_driver_state_t *netj )
     netj->srcaddress_valid = 0;
     if (netj->use_autoconfig)
     {
-	jacknet_packet_header *first_packet = alloca (sizeof (jacknet_packet_header));
-#ifdef WIN32
-    int address_size = sizeof( struct sockaddr_in );
-#else
-	socklen_t address_size = sizeof (struct sockaddr_in);
-#endif
-	//jack_info ("Waiting for an incoming packet !!!");
-	//jack_info ("*** IMPORTANT *** Dont connect a client to jackd until the driver is attached to a clock source !!!");
+        jacknet_packet_header *first_packet = alloca (sizeof (jacknet_packet_header));
+    #ifdef WIN32
+        int address_size = sizeof( struct sockaddr_in );
+    #else
+        socklen_t address_size = sizeof (struct sockaddr_in);
+    #endif
+        //jack_info ("Waiting for an incoming packet !!!");
+        //jack_info ("*** IMPORTANT *** Dont connect a client to jackd until the driver is attached to a clock source !!!");
 
-    while(1) {
-    first_pack_len = recvfrom (netj->sockfd, (char *)first_packet, sizeof (jacknet_packet_header), 0, (struct sockaddr*) & netj->syncsource_address, &address_size);
-#ifdef WIN32
-        if( first_pack_len == -1 ) {
-            first_pack_len = sizeof(jacknet_packet_header);
-            break;
+        while(1) {
+            if( ! netjack_poll( netj->sockfd, 1000 ) ) {
+                jack_info ("Waiting aborted");
+                return -1;
+            }
+            first_pack_len = recvfrom (netj->sockfd, (char *)first_packet, sizeof (jacknet_packet_header), 0, (struct sockaddr*) & netj->syncsource_address, &address_size);
+        #ifdef WIN32
+                if( first_pack_len == -1 ) {
+                    first_pack_len = sizeof(jacknet_packet_header);
+                    break;
+                }
+        #else
+                if (first_pack_len == sizeof (jacknet_packet_header))
+                    break;
+        #endif
         }
-#else
+        netj->srcaddress_valid = 1;
+
         if (first_pack_len == sizeof (jacknet_packet_header))
-            break;
-#endif
-    }
-	netj->srcaddress_valid = 1;
+        {
+            packet_header_ntoh (first_packet);
 
-	if (first_pack_len == sizeof (jacknet_packet_header))
-	{
-	    packet_header_ntoh (first_packet);
+            jack_info ("AutoConfig Override !!!");
+            if (netj->sample_rate != first_packet->sample_rate)
+            {
+                jack_info ("AutoConfig Override: Master JACK sample rate = %d", first_packet->sample_rate);
+                netj->sample_rate = first_packet->sample_rate;
+                }
 
-	    jack_info ("AutoConfig Override !!!");
-	    if (netj->sample_rate != first_packet->sample_rate)
-	    {
-		jack_info ("AutoConfig Override: Master JACK sample rate = %d", first_packet->sample_rate);
-		netj->sample_rate = first_packet->sample_rate;
-	    }
+            if (netj->period_size != first_packet->period_size)
+            {
+                jack_info ("AutoConfig Override: Master JACK period size is %d", first_packet->period_size);
+                netj->period_size = first_packet->period_size;
+                }
+            if (netj->capture_channels_audio != first_packet->capture_channels_audio)
+            {
+                jack_info ("AutoConfig Override: capture_channels_audio = %d", first_packet->capture_channels_audio);
+                netj->capture_channels_audio = first_packet->capture_channels_audio;
+            }
+            if (netj->capture_channels_midi != first_packet->capture_channels_midi)
+            {
+                jack_info ("AutoConfig Override: capture_channels_midi = %d", first_packet->capture_channels_midi);
+                netj->capture_channels_midi = first_packet->capture_channels_midi;
+            }
+            if (netj->playback_channels_audio != first_packet->playback_channels_audio)
+            {
+                jack_info ("AutoConfig Override: playback_channels_audio = %d", first_packet->playback_channels_audio);
+                netj->playback_channels_audio = first_packet->playback_channels_audio;
+                }
+            if (netj->playback_channels_midi != first_packet->playback_channels_midi)
+            {
+                jack_info ("AutoConfig Override: playback_channels_midi = %d", first_packet->playback_channels_midi);
+                netj->playback_channels_midi = first_packet->playback_channels_midi;
+            }
 
-	    if (netj->period_size != first_packet->period_size)
-	    {
-		jack_info ("AutoConfig Override: Master JACK period size is %d", first_packet->period_size);
-		netj->period_size = first_packet->period_size;
-	    }
-	    if (netj->capture_channels_audio != first_packet->capture_channels_audio)
-	    {
-		jack_info ("AutoConfig Override: capture_channels_audio = %d", first_packet->capture_channels_audio);
-		netj->capture_channels_audio = first_packet->capture_channels_audio;
-	    }
-	    if (netj->capture_channels_midi != first_packet->capture_channels_midi)
-	    {
-		jack_info ("AutoConfig Override: capture_channels_midi = %d", first_packet->capture_channels_midi);
-		netj->capture_channels_midi = first_packet->capture_channels_midi;
-	    }
-	    if (netj->playback_channels_audio != first_packet->playback_channels_audio)
-	    {
-		jack_info ("AutoConfig Override: playback_channels_audio = %d", first_packet->playback_channels_audio);
-		netj->playback_channels_audio = first_packet->playback_channels_audio;
-	    }
-	    if (netj->playback_channels_midi != first_packet->playback_channels_midi)
-	    {
-		jack_info ("AutoConfig Override: playback_channels_midi = %d", first_packet->playback_channels_midi);
-		netj->playback_channels_midi = first_packet->playback_channels_midi;
-	    }
-
-	    netj->mtu = first_packet->mtu;
-	    jack_info ("MTU is set to %d bytes", first_packet->mtu);
-	    netj->latency = first_packet->latency;
-	}
+            netj->mtu = first_packet->mtu;
+            jack_info ("MTU is set to %d bytes", first_packet->mtu);
+            netj->latency = first_packet->latency;
+        }
     }
     netj->capture_channels  = netj->capture_channels_audio + netj->capture_channels_midi;
     netj->playback_channels = netj->playback_channels_audio + netj->playback_channels_midi;
@@ -717,25 +767,25 @@ netjack_startup( netjack_driver_state_t *netj )
                              * 1000000.0f);
 
     if( netj->latency == 0 )
-	netj->deadline_offset = 50*netj->period_usecs;
+        netj->deadline_offset = 50*netj->period_usecs;
     else
-	netj->deadline_offset = netj->period_usecs + 10*netj->latency*netj->period_usecs/100;
+        netj->deadline_offset = netj->period_usecs + 10*netj->latency*netj->period_usecs/100;
 
     if( netj->bitdepth == CELT_MODE ) {
-	// celt mode.
-	// TODO: this is a hack. But i dont want to change the packet header.
-	netj->resample_factor = (netj->resample_factor * netj->period_size * 1024 / netj->sample_rate / 8)&(~1);
-	netj->resample_factor_up = (netj->resample_factor_up * netj->period_size * 1024 / netj->sample_rate / 8)&(~1);
+        // celt mode.
+        // TODO: this is a hack. But i dont want to change the packet header.
+        netj->resample_factor = (netj->resample_factor * netj->period_size * 1024 / netj->sample_rate / 8)&(~1);
+        netj->resample_factor_up = (netj->resample_factor_up * netj->period_size * 1024 / netj->sample_rate / 8)&(~1);
 
-	netj->net_period_down = netj->resample_factor;
-	netj->net_period_up = netj->resample_factor_up;
+        netj->net_period_down = netj->resample_factor;
+        netj->net_period_up = netj->resample_factor_up;
     } else {
-	netj->net_period_down = (float) netj->period_size / (float) netj->resample_factor;
-	netj->net_period_up = (float) netj->period_size / (float) netj->resample_factor_up;
+        netj->net_period_down = (float) netj->period_size / (float) netj->resample_factor;
+        netj->net_period_up = (float) netj->period_size / (float) netj->resample_factor_up;
     }
 
     netj->rx_bufsize = sizeof (jacknet_packet_header) + netj->net_period_down * netj->capture_channels * get_sample_size (netj->bitdepth);
-    global_packcache = packet_cache_new (netj->latency + 50, netj->rx_bufsize, netj->mtu);
+    netj->packcache = packet_cache_new (netj->latency + 50, netj->rx_bufsize, netj->mtu);
 
     netj->expected_framecnt_valid = 0;
     netj->num_lost_packets = 0;
@@ -745,9 +795,9 @@ netjack_startup( netjack_driver_state_t *netj )
 
     // Special handling for latency=0
     if( netj->latency == 0 )
-	netj->resync_threshold = 0;
+        netj->resync_threshold = 0;
     else
-	netj->resync_threshold = MIN( 15, netj->latency-1 );
+        netj->resync_threshold = MIN( 15, netj->latency-1 );
 
     netj->running_free = 0;
 
