@@ -1,20 +1,20 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 
@@ -40,7 +40,7 @@ class JackClientPipeThread : public JackRunnableInterface
         JackThread fThread;
         int fRefNum;
 
-        void ClientAdd(char* name, int pid, int* shared_engine, int* shared_client, int* shared_graph, int* result);
+        void ClientAdd(char* name, int pid, int uuid, int* shared_engine, int* shared_client, int* shared_graph, int* result);
         void ClientRemove();
         void ClientKill();
 
@@ -53,7 +53,7 @@ class JackClientPipeThread : public JackRunnableInterface
 
         int Open(JackServer* server);   // Open the Server/Client connection
         void Close();                   // Close the Server/Client connection
-   
+
         bool HandleRequest();
 
         // JackRunnableInterface interface
@@ -91,7 +91,7 @@ class JackWinNamedPipeServerChannel : public JackRunnableInterface
 
         int Open(const char* server_name, JackServer* server);  // Open the Server/Client connection
         void Close();                                           // Close the Server/Client connection
-    
+
         int Start();
 
         // JackRunnableInterface interface
